/*
 * gen4.c: A simple TCP benchmark application
 *
 * Based on orignal gen/sink code Copyright 1990,1991 by Les Roberson with
 * various modifications by other people at CERN.
 *
 * Copyright (c) 1999-2001 by Jes Sorensen, <Jes.Sorensen@cern.ch>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <errno.h>
#include <unistd.h>

#define oops(msg) {perror(msg); exit(-1);}

/* Report results after 10MB of data */
#define REPORT_FREQ 10*1024*1024
/*
 * Transfer data out of a 10MB buffer to make sure we are not just
 * transmitting data out of the hot cache.
 */
#define RECORD_BUFFER_SIZE 10*1024*1024

int max;
int maxseg,optlen;
int sndbuf;
int r;
extern int errno;

extern void report(), report_start(), report_stop();

struct hostent *hp;
FILE *sf;
int s,bytes, i, ch;
char urgent = 'Z';

#include <sys/ioctl.h>

int main(argc, argv)
int argc; 
char *argv[];
{
	struct sockaddr_in saddr;
	char *record_buffer, *orig_record_ptr;
	char reptext[256], *hostname;
	int portnum, rec_len, bytes_written;
	int i;

	/* check argument count */
	if(argc != 5) 
	{
		fprintf(stderr,"Usage: %s server_host server_port record_length setsockopt\n", argv[0]);
		exit(-1);
	}

	rec_len = atoi(argv[3]);

	if (rec_len > RECORD_BUFFER_SIZE) {
		fprintf(stderr, "rec_len (%i) exceeds maximum size %i\n",
			rec_len, RECORD_BUFFER_SIZE);
		exit(-1);
	}

	record_buffer = (char *)malloc(RECORD_BUFFER_SIZE + rec_len);
	if (!record_buffer) {
		fprintf(stderr, "Unable to allocate memory for transmit buffer\n");
		exit(-1);
	}
	orig_record_ptr = record_buffer;
	memset(record_buffer, 0, RECORD_BUFFER_SIZE);

	max = atoi(argv[4]);

	strcpy(reptext,"");
	/* get the record length */
	strncat(reptext, " reclen=", 255 - strlen(reptext));
	strncat(reptext, argv[3], 255 - strlen(reptext));

	/* extract the hostname and port number of the server */
	strncat(reptext, " TCPhost=", 255 - strlen(reptext));
	strncat(reptext, argv[1], 255 - strlen(reptext));
	hostname = argv[1];
	portnum = atoi(argv[2]);

	/* build the network address */
	bzero(&saddr,sizeof(saddr));
	saddr.sin_family = AF_INET;
	hp = gethostbyname(hostname);
	if (hp == NULL) 
		oops("no such host");
	bcopy(hp->h_addr,&saddr.sin_addr,hp->h_length);
	saddr.sin_port = htons(portnum);

	/* make the connection */
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s == -1) 
		oops("socket");

	if (setsockopt(s, SOL_SOCKET, SO_SNDBUF, (char *)&max, sizeof(max)) < 0)
		oops("setsockopt");
  
	printf("Setsockopt = %d\n",max);

	if (connect(s, &saddr, sizeof(saddr)) != 0)
		oops("connect");

	optlen = sizeof(maxseg);
	if (getsockopt(s,IPPROTO_TCP,TCP_MAXSEG,(char *)&maxseg,&optlen) < 0)
		oops("getsockopt MAXSEG");
  
	printf("Max seg TCP : %d\n",maxseg);

	/* initialise reporting */
	report_start(reptext,1,0,1,argv);
	report(1, "source", 0); 

	for (i = 1; i < 4000000; i++)
	{
		/* loop writing blocks */
		if ((bytes_written = write(s,record_buffer,rec_len)) < 0) 
			oops("write");
		if ((bytes = bytes + bytes_written) >= REPORT_FREQ) 
		{
			report(1, "source", bytes);
			bytes = 0;
		}
		record_buffer += rec_len;
		if ((unsigned long)record_buffer > (unsigned long)(orig_record_ptr + RECORD_BUFFER_SIZE))
			record_buffer = orig_record_ptr;
	}
	free(orig_record_ptr);
	return 0;
}
