/*
 * sink4.c: A simple TCP benchmark application
 *
 * Based on orignal gen/sink code Copyright 1990,1991 by Les Roberson with
 * various modifications by other people at CERN.
 *
 * Copyright (c) 1999-2001 by Jes Sorensen, <Jes.Sorensen@cern.ch>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <fcntl.h>
#include <signal.h>  
#include <errno.h>
#include <unistd.h>

#ifdef sgi 
#include <sys/ioctl.h>
#include <net/soioctl.h>
#endif

#ifdef __hpux
#include <fcntl.h>
#endif

/* Report results after 10MB of data */
#define REPORT_FREQ 10*1024*1024
/*
 * Receive data into a 10MB buffer to make sure we are not just
 * playing cache tricks.
 */
#define RECORD_BUFFER_SIZE 10*1024*1024

#define oops(msg) {perror(msg); exit(-1);}
int max;

extern void report(), report_start(), report_stop();

int maxseg,optlen;
int sndbuf;

struct sockaddr_in saddr;
struct hostent *hp;
FILE   *sf;
int    slen, s, sfd, bytes, bytes_read, i, ch;

int main(int argc, char *argv[])
{
	char *record_buffer, *orig_record_ptr;
	char reptext[256], *hostname;
	struct sigaction sa;
	int portnum, rec_len;
	int tape;
	int pid;

	/* check argument count */
	if (argc != 5) 
	{
		fprintf(stderr,"Usage: %s hostname server_port "
			"record_length setsockopt\n", argv[0]);
		exit(-1);       
	}

	/* Record length to use */
	rec_len = atoi(argv[3]);

	if (rec_len > RECORD_BUFFER_SIZE) {
		fprintf(stderr, "rec_len (%i) exceeds maximum size %i\n",
			rec_len, RECORD_BUFFER_SIZE);
		exit(-1);
	}

	record_buffer = (char *)malloc(RECORD_BUFFER_SIZE + rec_len);
	if (!record_buffer) {
		fprintf(stderr, "Unable to allocate memory for "
			"receive buffer\n");
		exit(-1);
	}
	orig_record_ptr = record_buffer;
	memset(record_buffer, 0, RECORD_BUFFER_SIZE);

	max = atoi(argv[4]);

	/* extract the port number to be used */
	portnum = atoi(argv[2]);

	strcpy(reptext,"");
	/* extract the record length */
	strncat(reptext, " reclen=", 255 - strlen(reptext));
	strncat(reptext, argv[3], 255 - strlen(reptext));

	/* build the network address */
	bzero(&saddr,sizeof(saddr));
	saddr.sin_family = AF_INET;
	hostname = argv[1];
	strncat(reptext, " TCPhost=", 255 - strlen(reptext));
	strncat(reptext, argv[1], 255 - strlen(reptext));
	hp = gethostbyname(hostname);
	if (hp == NULL) 
		oops("gethostbyname");
	bcopy(hp->h_addr,&saddr.sin_addr,hp->h_length);
	saddr.sin_port = htons(portnum);

	/* create the socket and bind the address */
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s == -1) 
		oops("socket");

	if (setsockopt(s, SOL_SOCKET, SO_RCVBUF, (char *)&max, sizeof(max)) < 0)
		oops("setsockopt");

	printf("Setsockopt rcv = %d\n",max);

	if (bind(s, &saddr, sizeof(saddr)) != 0)
		oops("bind");

	if (listen(s,5) != 0) 
		oops("listen");

	/* repeatedly wait for clients to connect */
	for (;;) 
	{
		sfd = accept(s, NULL, NULL);
		if (sfd == -1) 
			oops("accept");
      
		optlen = sizeof(maxseg);
		if (getsockopt(sfd,IPPROTO_TCP,TCP_MAXSEG,(char *)&maxseg,&optlen) < 0)
			oops("getsockopt MAXSEG");
		printf("Max seg TCP : %d\n",maxseg);

		/* initialise reporting */
		bytes = 0;
		report_start(reptext, 1, 0, 1, argv);
		report(1, "sink", 0);

		/* read data */  
		for (;;) 
		{
			if (!(bytes_read = read(sfd,record_buffer,rec_len))) 
				break;
	  
			if ((bytes = bytes+bytes_read) >= REPORT_FREQ ) 
			{
				report(1, "sink", bytes);
				bytes = 0;
			}
			record_buffer += rec_len;
			if ((unsigned long)record_buffer > (unsigned long)(orig_record_ptr + RECORD_BUFFER_SIZE))
				record_buffer = orig_record_ptr;
		}
		report(1, "sink", bytes);

		/* terminate reporting */
		report_stop();
	}
	return 0;
}
