/*
 * tub4.c: A simple TCP benchmark application
 *
 * Based on orignal gen/sink code Copyright 1990,1991 by Les Roberson with
 * various modifications by other people at CERN.
 *
 * Copyright (c) 1999-2001 by Jes Sorensen, <Jes.Sorensen@cern.ch>.
 * Copyright (c) 2001 by Sandro Mazzucato <sandro@zeroknowledge.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <fcntl.h>
#include <signal.h>  
#include <errno.h>
#include <sys/poll.h>
#ifdef sgi 
#include <sys/ioctl.h>
#include <net/soioctl.h>
#endif
#include <malloc.h>
#include <unistd.h>

#ifdef __hpux
#include <fcntl.h>
#endif

/* Report results after 10MB of data */
#define REPORT_FREQ 10*1024*1024
/*
 * Receive data into a 10MB buffer to make sure we are not just
 * playing cache tricks.
 */
#define RECORD_BUFFER_SIZE 10*1024*1024

/*
 * Max number of clients that we allow to connect simultaneously
 */
#define MAX_CLIENTS 10

#define oops(msg) {perror(msg); exit(-1);}
int max;

extern void report(), report_start(), report_stop();

int maxseg,optlen;
int sndbuf;

struct sockaddr_in saddr;
struct hostent *hp;
FILE   *sf;
int    slen, s, sfd, bytes, bytes_read, i, ch;

int main(int argc, char *argv[])
{
  char *record_buffer, *orig_record_ptr;
  char reptext[256], *hostname;
  int portnum, rec_len;
  struct pollfd pfd[MAX_CLIENTS+1];
  int num_clients = 0, num, i,j;
        
  /* check argument count */
  if (argc != 5) 
    {
      fprintf(stderr,"Usage: %s hostname server_port "
              "record_length setsockopt\n", argv[0]);
      exit(-1);       
    }

  /* Record length to use */
  rec_len = atoi(argv[3]);

  if (rec_len > RECORD_BUFFER_SIZE) {
    fprintf(stderr, "rec_len (%i) exceeds maximum size %i\n",
            rec_len, RECORD_BUFFER_SIZE);
    exit(-1);
  }

  record_buffer = (char *)malloc(RECORD_BUFFER_SIZE + rec_len);
  if (!record_buffer) {
    fprintf(stderr, "Unable to allocate memory for "
            "receive buffer\n");
    exit(-1);
  }
  orig_record_ptr = record_buffer;
  memset(record_buffer, 0, RECORD_BUFFER_SIZE);

  max = atoi(argv[4]);

  /* extract the port number to be used */
  portnum = atoi(argv[2]);

  strcpy(reptext,"");
  /* extract the record length */
  strncat(reptext, " reclen=", 255 - strlen(reptext));
  strncat(reptext, argv[3], 255 - strlen(reptext));

  /* build the network address */
  bzero(&saddr,sizeof(saddr));
  saddr.sin_family = AF_INET;
  hostname = argv[1];
  strncat(reptext, " TCPhost=", 255 - strlen(reptext));
  strncat(reptext, argv[1], 255 - strlen(reptext));
  hp = gethostbyname(hostname);
  if (hp == NULL) 
    oops("gethostbyname");
  bcopy(hp->h_addr,&saddr.sin_addr,hp->h_length);
  saddr.sin_port = htons(portnum);

  /* create the socket and bind the address */
  s = socket(AF_INET, SOCK_STREAM, 0);
  if (s == -1) 
    oops("socket");

  if (setsockopt(s, SOL_SOCKET, SO_RCVBUF, (char *)&max, sizeof(max)) < 0)
    oops("setsockopt");

  printf("Setsockopt rcv = %d\n",max);

  if (bind(s, &saddr, sizeof(saddr)) != 0)
    oops("bind");

  if (listen(s,5) != 0) 
    oops("listen");

  /* Init poll */
  pfd[num_clients].fd = s;
  pfd[num_clients].events = POLLIN;
  num_clients++;
  /* repeatedly wait for clients to connect */
  /* initialise reporting */
  bytes = 0;
  report_start(reptext, 1, 0, 1, argv);
  report(1, "sink", 0);
  for (;;) 
    {
      if ( (num = poll(pfd, num_clients, -1)) < 0 ) {
        oops("poll");
      }

      for ( j= 0,i = 0; i < num_clients && j < num ; i++ ) {

        if ( pfd[i].revents ) {
          j++;
          if ( i == 0 ) { /* New connection */
            sfd = accept(pfd[i].fd, NULL, NULL);
            if (sfd == -1) 
              oops("accept");
      
            optlen = sizeof(maxseg);
            if (getsockopt(sfd,IPPROTO_TCP,TCP_MAXSEG,(char *)&maxseg,&optlen) < 0)
              oops("getsockopt MAXSEG");
            printf("Max seg TCP : %d\n",maxseg);
            if ( num_clients == MAX_CLIENTS+1 ) {
              close(sfd);
            } else {
              pfd[num_clients].fd = sfd;
              pfd[num_clients].events = POLL_IN;
              num_clients++;
            }
          } else {
              
            /* read data */  
            if (!(bytes_read = read(pfd[i].fd,record_buffer,rec_len))) 
              break;
	  
            if ((bytes = bytes+bytes_read) >= REPORT_FREQ ) 
              {
                report(1, "tub", bytes);
                bytes = 0;
              }
            record_buffer += rec_len;
            if ((unsigned long)record_buffer > (unsigned long)(orig_record_ptr + RECORD_BUFFER_SIZE))
              record_buffer = orig_record_ptr;
          }
        }
      }
    }
  report(1, "sink", bytes);

  /* terminate reporting */
  report_stop();
}
